
// QTreeWindow: the main widget for P4QTree. 

#ifndef QTREEWINDOW_H

#include <clientapi.h>
#include <qbitmap.h>
#include <qcanvas.h>
#include <qlabel.h>
#include <qmenubar.h>
#include <qtooltip.h>

#include "changesorter.h"
#include "filehead.h"
#include "filelogcache.h"

#include "qtreeitem.h"

#include "clientjobdescuser.h"
#include "qtreejobtext.h"
#include "qtreejobbar.h"

#include "clientchangeuser.h"
#include "qtreechangenum.h"
#include "qtreechangebar.h"

#include "qtreerevtext.h"
#include "qtreerevball.h"
#include "qtreefiletext.h"
#include "qtreefilestripe.h"
#include "qtreearrow.h"
#include "qtreecanvas.h"

#include "qtreetooltip.h"

#include "qtreeprefsdialog.h"
#include "qtreelegenddialog.h"

extern bool console;

#endif // QTREEWINDOW_H

#include "qtreeeasteregg.h"

class QTreeChangeView;

class QTreeWindow : public QCanvasView 
{
	Q_OBJECT
public:
	// Arguments to the constructor:
	//  parent: the parent window (NULL if this is the main window)
	//  depotfile: name of a Perforce file
	//  client: a ClientApi with settings already set
	//  error: any old blank Error
	//  showjobs: should the job bar be created?
	//  isbirds: is this a branching-only window?
	//  wflags: normal widget flags
	 
	QTreeWindow( QWidget* parent, char* depotfile, 
		ClientApi* client, Error* error, bool isbirds, char* cfile );
	virtual ~QTreeWindow();

	// Set maximum size, tend to scrollbars, et cetera. 
	void AdjustGeometry();

	// Center view on head rev of "main" file 
	void CenterOnMain();

	bool IsEmpty() 
	{ 
		return 
		( !qtcanvas || qtcanvas->IsEmpty() || !filelabel || !changeview ); 
	} ;

	// Spit contents of "error" to a dialog. 
	void HandleError();
	int HandleFatalError();

	// Load preferences from QSettings.
	void LoadSettings();

	// Save image directly to filename.png.
	void SaveImage( QString filename );

	ClientApi* client;
	Error* error;
	QMenuBar* menubar;
	char* cachefile;
protected:
	// Intercept events and AdjustGeometry() before handling them. 
	bool event( QEvent* e );

	// Drag 'n drop. 
	void contentsMousePressEvent( QMouseEvent* e );
	void contentsDragEnterEvent( QDragEnterEvent* e );
	void contentsDropEvent( QDropEvent* e );

	// Double-click. 
	void contentsMouseDoubleClickEvent( QMouseEvent* e );

	// Overriden method to give space to the menu bar. 
	void setVBarGeometry ( QScrollBar & vbar, int x, int y, int w, int h );

	void keyPressEvent( QKeyEvent* e ) { egg->KeyPress( e->key() ); } ;
public slots:
	// Scroll filetexts. 
	void HorizScroll( int x, int y );

	// Pop up an About dialog. 
	void AboutDialog();
	// Pop up the -h "usage" dialog. 
	static void UsageDialog( QWidget* parent=NULL );
	// Save canvas to a PNG. 
	void SaveImage();
	// Print canvas.
	void PrintImage();
	// Pop up a miniature branch-only window. 
	void SpawnMini();
	// Toggle job bar on and off. 
	void ToggleJobs();
	// Preferences dialog.
	void Preferences();
	// Legend dialog
	void Legend();
	// Refresh
	void Refresh();
private:
	// init methods
	void InitMenus();
	void InitIcons();

	// basic contents
	QTreeCanvas* qtcanvas;
	QTreeChangeView* changeview;

	// settings
	char* mainfile;
	bool birdseyeview;
	bool havejobbar;
	bool jobsvisible;
	bool showall;
	QString startdate, startchange, endcd;
	bool binary; //do binary diffs?

	QTreeWindow* minime;		// child "branch view" window
	QDialog* legend;			// child "legend" window
	QLabel* filelabel;			// widget at left of window
	QCanvasItem* dragsource;	// source of current drag

	// extra attachments
	QPixmap appicon;
	QTreeToolTip tips;
	QTreeEasterEgg* egg;
};
